<?php
// tòmò web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This file is derived from:
// UNB Components
// Copyright since 2007 by Yves Goergen
// Website: http://unclassified.de/go/unb2
//
// This file contains the UnbConfiguration class.

if (!class_exists('UnbDatabase', false)) require_once __DIR__ . '/database.class.php';

// Configuration functions for reading and writing UNB settings.
//
class UnbConfiguration
{
	// Configuration filename. If it's not an absolute path, the name is interpreted relative to this class file.
	private static $settingsFilename = 'config.inc.php';
	// Settings cache array
	private static $settings = null;

	// Private constructor, does nothing but preventing an unwanted instantiation of this class.
	//
	private function __construct()
	{
	}

	// Sets the configuration file name.
	//
	// If the specified file name is not an absolute path, the name is interpreted relative to this
	// class source file. The default file name is "config.inc.php" in this directory.
	//
	// filename = (string) Name of the configuration file, or {{null}} or empty to not read any file
	//
	public static function SetFilename($filename)
	{
		if (isset($filename))
		{
			if (!is_string($filename))
				throw new InvalidArgumentException('Invalid argument type: $filename');
			$filename = trim($filename);
			if (!strlen($filename))
				$filename = null;
		}

		self::$settingsFilename = $filename;
	}

	// Reads the configuration file.
	//
	// The configuration filename is specified by the $settingsFilename variable. The file will
	// simply be included by PHP so it must be a correct and valid PHP source file. Its only task
	// is to define some values in the $settings array. All values defined in that file will be
	// copied to the UnbConfiguration local cache array. Using a normal PHP source file with its
	// default filename extension ".php" has the advantage that it cannot be read by a web browser
	// which would reveal secret information like a database password.
	//
	// You can also initialise the settings by invoking Initialise instead of having them read from
	// a separate PHP source file.
	//
	private static function ReadFile()
	{
		if (isset(self::$settingsFilename))
		{
			if (self::$settingsFilename{0} == '/')
				include self::$settingsFilename;
			else
				include __DIR__ . '/' . self::$settingsFilename;

			if (!is_array($settings))
				throw new Exception('[UNB#1] Configuration file is corrupt.');
			self::$settings = $settings;
		}
	}

	// Initialises the configuration data from code.
	//
	// This function does the same as ReadFile, only that it reads the settings array from a
	// parameter instead of a separate PHP source file.
	//
	// settings = (array(key => value)) Configuration settings as they can be stored in a separate file
	//
	public static function Initialise($settings)
	{
		if (!is_array($settings))
			throw new InvalidArgumentException('Invalid argument type: $settings');

		self::$settings = $settings;
	}

	// Gets a configuration setting.
	//
	// name = (string) Setting name to get (max. 255 characters)
	// default = (any) Default return value if no setting was found. This is only used if the name contains no wildcard character.
	//
	// You can get settings by wildcards: Put a * for multiple possible characters.
	//
	// Throws InvalidArgumentException
	// Throws LengthException
	// Throws UnbDatabaseException
	// Throws Exception if the configuration file is corrupt.
	//
	// Returns (string) configuration setting value or (array(name => data)) if fed with a wildcard
	//
	// See UnbUser::GetSettings()
	//
	public static function Get($name, $default = null)
	{
		if (!isset(self::$settings))
			self::ReadFile();

		$entries = array();
		if (strpos($name, '*') === false)
		{
			if (array_key_exists($name, self::$settings))
				return self::$settings[$name];
		}
		else
		{
			$rsName = preg_replace('_([\\\\^$.\\]|()?+{}\\_])_', '\\\\$1', $name);   // regexp-safe name
			$rsName = str_replace('*', '.*', $rsName);
			foreach (self::$settings as $myname => $data)
			{
				if (preg_match('_^' . $rsName . '$_i', $myname))
					$entries[$myname] = $data;
			}
		}

		// Only return an array if fed with a wildcard
		if (strpos($name, '*') === false)
		{
			if (count($entries) > 0)
				return $entries[0];
			else
				return $default;
		}
		else
		{
			return $entries;
		}
	}

} // class UnbConfiguration

?>
