<?php
// tòmò web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This file contains the PublicSuffix class.

class PublicSuffix
{
	public static $dataFileName;
	
	private static $publicSuffixData;
	
	public static function GetSuffixDataFileName()
	{
		$fileName = self::$dataFileName;
		if (!isset($fileName))
		{
			return __DIR__ . '/publicsuffixdata.inc.php';
		}
		return $fileName;
	}
	
	// Gets the shortest regularly registered domain name out of a given full host name.
	//
	// hostName = (string) Full host name to process, must be in IDN Punycode form (i.e. not Unicode UTF-8).
	//
	// Returns (string) Domain name that can be used as provider name, or null for invalid host names.
	//
	public static function GetPublicSuffix($hostName)
	{
		if (!isset(self::$publicSuffixData))
		{
			include_once self::GetSuffixDataFileName();
			self::$publicSuffixData = $publicSuffixData;
			if (!isset(self::$publicSuffixData))
			{
				return false;
			}
		}
		
		// Clean up given host name
		$hostName = strtolower($hostName);
		$hostName = trim($hostName, '.');
		$hostName = explode('.', $hostName);
		$tld = $hostName[count($hostName) - 1];

		// Compare both types of rules, exceptions first, then normal (positive) rules
		for ($type = 0; $type <= 1; $type++)
		{
			// Compare every rule
			foreach (self::$publicSuffixData[$type] as $rule)
			{
				if ($tld !== $rule[count($rule) - 1]) continue;   // Top-level domain does not match
				if (count($rule) > count($hostName)) continue;   // Rule is longer than host name
				// Compare every segment of the rule
				for ($i = 0; $i < count($rule); $i++)
				{
					$hnPart = $hostName[count($hostName) - 1 - $i];
					$rPart = $rule[count($rule) - 1 - $i];
					if ($rPart !== '*' &&
						$rPart !== $hnPart) continue 2;   // Some non-wildcard segment of the rule did not match
				}
				// All rule segments matched, finish up
				$isException = $type == 0;
				if (!$isException && count($rule) == count($hostName)) return null;   // Host name is exactly the rule
				// Combine as many parts as the rule's length plus one more as the resulting domain name
				return join('.', array_slice($hostName, -count($rule) - 1 + ($isException ? 1 : 0)));
			}
		}
		return null;
	}

	public static function GenerateDataFile()
	{
		// Download the current file from Mozilla's code repository or use local copy
		if (file_exists('effective_tld_names.dat'))
			$lines = file('effective_tld_names.dat');
		else
			$lines = file('http://mxr.mozilla.org/mozilla-central/source/netwerk/dns/effective_tld_names.dat?raw=1');

		$exceptionRules = array();
		$normalRules = array();

		foreach ($lines as $rule)
		{
			// Ignore empty or comment lines
			$rule = trim($rule);
			if (!strlen($rule)) continue;
			if (substr($rule, 0, 2) == '//') continue;

			// Detect exception rules
			$exception = false;
			if ($rule{0} === '!')
			{
				$exception = true;
				$rule = substr($rule, 1);
			}

			// Convert everything from UTF-8 to IDN Punycode (requires PHP 5.3 and intl extension)
			$rule = idn_to_ascii($rule);

			// Split to levels
			$rule = explode('.', $rule);
			$levels = count($rule);

			if ($exception)
			{
				$exceptionRules[$levels][] = $rule;
			}
			else
			{
				$normalRules[$levels][] = $rule;
			}
		}

		$publicSuffixData = array();
		// $publicSuffixData[0] contains all exceptions
		// $publicSuffixData[1] contains all normal rules

		for ($i = 10; $i >= 1; $i--)
		{
			if (is_array($exceptionRules[$i]))
			{
				foreach ($exceptionRules[$i] as $rule)
				{
					$publicSuffixData[0][] = $rule;
				}
			}
		}
		for ($i = 10; $i >= 1; $i--)
		{
			if (is_array($normalRules[$i]))
			{
				foreach ($normalRules[$i] as $rule)
				{
					$publicSuffixData[1][] = $rule;
				}
			}
		}

		$out = "<?php\n" .
			"// This Source Code Form is subject to the terms of the Mozilla Public\n" .
			"// License, v. 2.0. If a copy of the MPL was not distributed with this\n" .
			"// file, You can obtain one at http://mozilla.org/MPL/2.0/.\n\n" .
			"// Source: http://publicsuffix.org/\n\n" .
			"// Generated by tòmò web analytics, (c) Yves Goergen.\n\n" .
			"\$publicSuffixData=" . self::GenerateArrayCode($publicSuffixData) . ";\n" .
			"?>";
		file_put_contents(self::GetSuffixDataFileName(), $out);
	}

	private static function GenerateArrayCode(&$arr)
	{
		$out = 'array(';
		$count = 0;
		foreach ($arr as $a)
		{
			if ($count++)
			{
				$out .= ',';
			}
			if (is_array($a))
			{
				$out .= self::GenerateArrayCode($a);
			}
			else if (is_string($a))
			{
				$out .= '\'' . addslashes($a) . '\'';
			}
			else
			{
				$out .= $a;
			}
		}
		$out .= ')';
		return $out;
	}

} // class PublicSuffix

?>
